"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getBoundary = void 0;
const stream_1 = require("stream");
const queue_1 = __importDefault(require("./queue"));
const Char = {
    BOUNDARY: 45,
    RETURN: 13,
    NEW_LINE: 10,
};
const HEAD = new Uint8Array([
    Char.RETURN,
    Char.NEW_LINE,
]);
const TAIL = new Uint8Array([
    Char.BOUNDARY,
    Char.BOUNDARY,
    Char.RETURN,
    Char.NEW_LINE,
]);
const SEPARATOR = new Uint8Array([
    Char.RETURN,
    Char.NEW_LINE,
    Char.RETURN,
    Char.NEW_LINE,
]);
const factory = (name, config) => {
    let length = 0;
    const queue = (0, queue_1.default)();
    const boundary = new Uint8Array(('--' + name)
        .split('')
        .map((char) => char.charCodeAt(0)));
    const data = [];
    let landmark;
    let start;
    const end = Buffer.from(TAIL);
    const read = (queue) => {
        if (!landmark) {
            if (!queue.until(boundary)) {
                return null;
            }
        }
        if (!landmark) {
            landmark = queue.slice(0);
        }
        if (!start) {
            start = Buffer.concat([
                HEAD,
                landmark,
            ]);
        }
        while (queue.until(start)) {
            const block = queue.slice((start.length) * -1);
            queue.slice(start.length);
            data.push(block);
        }
        if (queue.is(end)) {
            return data;
        }
        return null;
    };
    const extract = (blocks, push) => {
        for (const block of blocks) {
            const queue = (0, queue_1.default)();
            queue.concat(block);
            queue.slice(2);
            queue.until(SEPARATOR);
            const header = queue.slice(-4);
            queue.slice(4);
            const body = queue.get();
            push([
                split(header),
                body,
            ]);
        }
    };
    const split = (buffer) => (buffer.toString()
        .split(/\r\n/g)
        .map((line) => line.split(/:\s/))
        .reduce((acc, [name, ...rest]) => ((acc[name] = rest.join(': ')) && acc), {}));
    function transform(chunk, encoding, callback) {
        length += chunk.length;
        if ((config === null || config === void 0 ? void 0 : config.limit) && length > config.limit) {
            callback(new Error(`limit of ${config.limit} bytes exceeded`));
            return;
        }
        queue.concat(chunk);
        const blocks = read(queue);
        if (blocks) {
            extract(blocks, this.push.bind(this));
        }
        callback();
    }
    return new stream_1.Transform({
        objectMode: true,
        transform,
    });
};
const getBoundary = (contentType) => {
    if (!contentType.match(/multipart\/form-data/i)) {
        return null;
    }
    const match = contentType.match(/boundary=--[-?]*(\w+);?/);
    if (!match) {
        return null;
    }
    return match.pop();
};
exports.getBoundary = getBoundary;
exports.default = factory;
